package com.example.projekt4;

import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import com.example.projekt4.databinding.FragmentFirstBinding;
import com.example.waluty.ExchangeRatesTable;
import com.example.waluty.ObslugaNBP;
import com.example.waluty.Rate;
import com.example.waluty.SingletonPobranaTabela;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/* Ważnym aspektem tego przykładu jest pobieranie danych z sieci, które trzeba wykonać w tle,
   aby nie blokować głównego wątku obsługującego interfejs użytkownika.
   Gdybyśmy pobieranie wykonali bezpośrednio w procedurze obsługi zdarzenia, aplikacja
   byłaby nieresponsywna w trakcie tego pobierania.
   Android sam wykrywa taką sytuację i przerywa pobieranie wyjątkiem.

   Na potrzeby pobierania w tle przygotowujemy dodatkowy wątek w formie tzw. Executora
   - jest to rozwiązanie, które można łatwo skalować zwiększając liczbę wątków w puli.
   U nas będzie to jednak pojedynczy dodatkowy wątek.
 */
public class FirstFragment extends Fragment {
    private final ExecutorService watkiPobierajace = Executors.newSingleThreadExecutor();
    // private ExecutorService watkiPobierajace = Executors.newFixedThreadPool(4);

    private FragmentFirstBinding binding;

    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState
    ) {
        binding = FragmentFirstBinding.inflate(inflater, container, false);
        return binding.getRoot();

    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        binding.buttonFirst.setOnClickListener(v ->
                NavHostFragment.findNavController(FirstFragment.this)
                        .navigate(R.id.action_FirstFragment_to_SecondFragment)
        );
        // jeśli waluty były już pobierane - wykorzystaj dane zapisane w pamięci
        ExchangeRatesTable table = SingletonPobranaTabela.getInstance().getTable();
        if(table != null) {
            odswiezWidok(table);
        }
        // do przycisku "Pobierz bieżące" przypisujemy akcję pobierania
        binding.buttonPobierzBiezace.setOnClickListener(v -> {
            wykonajCalePobieranie();
        });
        // TODO dla chętnych - dodaj operację pobierania kursów archiwalnych w oparciu o datę
    }

    private void wykonajCalePobieranie() {
        Log.d("Waluty", "zlecam pobieranie");
        // zlecam zadanie do wykonania w tle
        watkiPobierajace.submit(() -> {
            Log.d("Waluty", "executor: początek pobierania");
            ExchangeRatesTable table = ObslugaNBP.pobierzBiezaceKursy();
            SingletonPobranaTabela.getInstance().setTable(table); // aby SecondFragment też mógł korzystać...
            Log.d("Waluty", "executor: koniec pobierania");
            // jednak zmiany w wyglądzie UI powinny być wykonane przez wątek główny
            // - zlecam wątkowi głównemu wykonanie "gdy znajdzie na to czas"
            this.getActivity().runOnUiThread(() -> {
                odswiezWidok(table);
            });
        });
        Log.d("Waluty", "pobieranie zlecone");
    }

    private void odswiezWidok(ExchangeRatesTable table) {
        Log.d("Waluty", "odswiezWidok");
        // gdy mamy binding, nie trzeba już find
        TextView textViewStatus = binding.textViewStatus;
        LinearLayout ll = binding.linearLayoutWaluty;
        if(table == null) {
            textViewStatus.setText(R.string.brak_danych);
        } else {
            textViewStatus.setText(table.toString());
            ll.removeAllViews();
            for(Rate rate : table.getRates()) {
                TextView textView = new TextView(this.getContext());
                textView.setText(rate.toString());
                ll.addView(textView);
            }
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

}