package com.example.projekt4;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Spinner;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import com.example.projekt4.databinding.FragmentSecondBinding;
import com.example.waluty.ExchangeRatesTable;
import com.example.waluty.Rate;
import com.example.waluty.SingletonPobranaTabela;

import java.math.BigDecimal;

public class SecondFragment extends Fragment {
    private FragmentSecondBinding binding;
    private String[] codes = {};
    private KtoraKwota ktoraKwotaBylaWpsiana = null;
    private ExchangeRatesTable table;
    private Rate selectedRate;

    private enum KtoraKwota {WALUTA, PLN};

    @Override
    public View onCreateView(
            @NonNull LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState
    ) {

        binding = FragmentSecondBinding.inflate(inflater, container, false);
        return binding.getRoot();

    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        table = SingletonPobranaTabela.getInstance().getTable();
        codes = table != null ? table.getCodes() : new String[0];
        Spinner spinner = binding.spinnerWyborWalut;
        ArrayAdapter<String> adapter = new ArrayAdapter<>(this.getContext(), android.R.layout.simple_spinner_item, codes);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);

        binding.buttonSecond.setOnClickListener(v ->
                NavHostFragment.findNavController(SecondFragment.this)
                        .navigate(R.id.action_SecondFragment_to_FirstFragment)
        );

        binding.spinnerWyborWalut.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                selectedRate = table.find(codes[position]);
                odwiezWidok();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                selectedRate = null;
                odwiezWidok();
            }
        });

        // zdarzenia po wpisaniu liczby w górnym lub dolnym polu i zatwierdzeniu
        // (na komputerze to "naciśnięcie enter")
        binding.editTextKwota1.setOnClickListener(v -> {
            ktoraKwotaBylaWpsiana = KtoraKwota.WALUTA;
            odwiezWidok();
        });
        binding.editTextKwota2.setOnClickListener(v -> {
            ktoraKwotaBylaWpsiana = KtoraKwota.PLN;
            odwiezWidok();
        });
    }

    private void odwiezWidok() {
        binding.textViewCurrencyName.setText(selectedRate == null ? "" : selectedRate.getCurrency());
        binding.labelKwota1.setText(selectedRate == null ? "Kwota" : ("Kwota " + selectedRate.getCode()));
        if(selectedRate == null || ktoraKwotaBylaWpsiana == null) {
            binding.editTextKwota1.setText("");
            binding.editTextKwota2.setText("");
            return;
        }
        switch (ktoraKwotaBylaWpsiana) {
            case WALUTA: {
                BigDecimal kwota1 = new BigDecimal(binding.editTextKwota1.getText().toString().replace(',', '.'));
                BigDecimal kwota2 = selectedRate.przeliczNaZlote(kwota1);
                binding.editTextKwota2.setText(String.valueOf(kwota2));
                break;
            }
            case PLN: {
                BigDecimal kwota2 = new BigDecimal(binding.editTextKwota2.getText().toString().replace(',', '.'));
                BigDecimal kwota1 = selectedRate.przeliczNaWalute(kwota2);
                binding.editTextKwota1.setText(String.valueOf(kwota1));
                break;
            }
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

}