package com.example.waluty;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import java.math.BigDecimal;

public class RateTest {
    public static final BigDecimal KURS_DOLARA = new BigDecimal("4.0000");
    private Rate rate1;

    @Before
    public void setUp() {
        rate1 = new Rate("USD", "dolar", KURS_DOLARA);
    }

    @Test
    public void testGetters() {
        assertEquals("USD", rate1.getCode());
        assertEquals("dolar", rate1.getCurrency());
        assertEquals(KURS_DOLARA, rate1.getMid());
    }

    @Test
    public void testToString() {
        assertEquals("USD (dolar): 4.0000", rate1.toString());
    }

    @Test
    public void testPrzeliczNaZlote() {
        BigDecimal kwota = new BigDecimal("1000.00");
        BigDecimal oczekiwanyWynik = new BigDecimal("4000.00");
        BigDecimal wynik = rate1.przeliczNaZlote(kwota);
        // sprawdzenie równości "technicznej", uwzględnia także informację o precyzji
        assertEquals(oczekiwanyWynik, wynik);

        // sprawdzenie róności matematycznej - nie bierze pod uwagę precyzji:
        // assert oczekiwanyWynik.compareTo(wynik) == 0;
    }


    @Test
    public void testPrzeliczNaZlote2() {
        Rate euro = new Rate("EUR", "euro", new BigDecimal("4.3333"));
        BigDecimal kwota = new BigDecimal("2.00");
        BigDecimal wynik = euro.przeliczNaZlote(kwota);
        BigDecimal oczekiwanyWynik = new BigDecimal("8.67");

        assertEquals(oczekiwanyWynik, wynik);
    }

    @Test
    public void testPrzeliczNaWalute() {
        BigDecimal kwota = new BigDecimal("4000.00");
        BigDecimal wynik = rate1.przeliczNaWalute(kwota);
        BigDecimal oczekiwanyWynik = new BigDecimal("1000.00");
        assertEquals(oczekiwanyWynik, wynik);
    }

    @Test
    public void testPrzeliczNaWaluteZaokraglanie() {
        Rate trzy = new Rate("TRI", "waluta trzy", new BigDecimal("3.0000"));
        BigDecimal kwota = new BigDecimal("2000.00");
        BigDecimal wynik = trzy.przeliczNaWalute(kwota);
        BigDecimal oczekiwanyWynik = new BigDecimal("666.67");
        assertEquals(oczekiwanyWynik, wynik);
    }

}